// -*- C++ -*-
/*!
 * @file  Kinect.h
 * @brief RTC Kinect 4 Windows
 * @date  $Date$
 *
 * $Id$
 */

#ifndef KINECT_H
#define KINECT_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "KinectDataTypeStub.h"
#include "ImgStub.h"

// </rtc-template>

using namespace RTC;

struct INuiSensor;
/*!
 * @class Kinect
 * @brief RTC Kinect 4 Windows
 *
 */
class Kinect
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  Kinect(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~Kinect();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * 
   * - Name:  debug
   * - DefaultValue: 0
   */
  int m_debug;
  /*!
   * 
   * - Name:  enable_camera
   * - DefaultValue: 1
   */
  std::string m_enable_camera;
  /*!
   * 
   * - Name:  enable_depth
   * - DefaultValue: 1
   */
  std::string m_enable_depth;
  /*!
   * 
   * - Name:  image_size
   * - DefaultValue: 480
   */
  std::string m_image_size;
  /*!
  *
  * - Name:  depth_width
  * - DefaultValue: 320
  */
  std::string m_depth_size;
  /*!
   * 
   * - Name:  player_index
   * - DefaultValue: 0
   */
  int m_player_index;
  /*!
   * 
   * - Name:  kinect_index
   * - DefaultValue: 0
   */
  int m_kinect_index;
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  RTC::TimedLong m_targetElevation;
  /*!
   */
  InPort<RTC::TimedLong> m_targetElevationIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
#ifdef OLD_IMAGE
  RTC::CameraImage m_image;
  /*!
   * Camera Color Image
   * - Type: CameraImage
   */
  OutPort<RTC::CameraImage> m_imageOut;
#else
  Img::TimedCameraImage m_image;
  /*!
  * Camera Color Image
  * - Type: TimedCameraImage
  */
  OutPort<Img::TimedCameraImage> m_imageOut;
#endif

  KINECT::DepthImage m_depth;
  /*!
   * Depth Image
   * - Type: CameraImage
   */
  OutPort<KINECT::DepthImage> m_depthOut;
  RTC::TimedLong m_currentElevation;
  /*!
   */
  OutPort<RTC::TimedLong> m_currentElevationOut;
  KINECT::NuiSkeletonFrame m_skeleton;
  /*!
   */
  OutPort<KINECT::NuiSkeletonFrame> m_skeletonOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>
  
  	HANDLE m_pVideoStreamHandle;
	HANDLE m_pDepthStreamHandle;

	HRESULT WriteColorImage();
	HRESULT WriteDepthImage();
	HRESULT WriteElevation();
	HRESULT WriteSkeleton();

	INuiSensor *    m_pNuiSensor;

	int m_depth_width, m_depth_height;
	int m_image_width, m_image_height;

};


extern "C"
{
  DLL_EXPORT void KinectInit(RTC::Manager* manager);
};

#endif // KINECT_H
