// -*- C++ -*-
/*!
 * @file  ManipulatorControler.cpp
 * @brief ModuleDescription
 * @date $Date$
 *
 * $Id$
 */

#include "ManipulatorControler.h"
#include <math.h>

// Module specification
// <rtc-template block="module_spec">
static const char* manipulatorcontroler_spec[] =
  {
    "implementation_id", "ManipulatorControler",
    "type_name",         "ManipulatorControler",
    "description",       "ModuleDescription",
    "version",           "1.0.0",
    "vendor",            "VenderName",
    "category",          "Category",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    ""
	// Configuration variables
	"conf.default.Speed", "20",

	// Widget
	"conf.__widget__.Speed", "text",
	// Constraints

	"conf.__type__.Speed", "int",

	""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
ManipulatorControler::ManipulatorControler(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_PointIn("Point", m_Point),
    m_HandIn("Hand", m_Hand),
    m_ManipulationCommonPort("ManipulationCommon"),
    m_ManipulationMiddlePort("ManipulationMiddle")

    // </rtc-template>
	
{
}

/*!
 * @brief destructor
 */
ManipulatorControler::~ManipulatorControler()
{
}



RTC::ReturnCode_t ManipulatorControler::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("Point", m_PointIn);
  addInPort("Hand", m_HandIn);
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  m_ManipulationCommonPort.registerConsumer("ManipulationCommon", "JARA_ARM::ManipulatorCommonInterface_Common", m_ManipulationCommon);
  m_ManipulationMiddlePort.registerConsumer("ManipulationMiddle", "JARA_ARM::ManipulatorCommonInterface_Middle", m_ManipulationMiddle);
  
  // Set CORBA Service Ports
  addPort(m_ManipulationCommonPort);
  addPort(m_ManipulationMiddlePort);
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  bindParameter("Speed", m_Speed, "20");
  // </rtc-template>
  
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ManipulatorControler::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ManipulatorControler::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ManipulatorControler::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t ManipulatorControler::onActivated(RTC::UniqueId ec_id)
{
  //voC_xActivate邽
  /*sleep(1);*/

	m_rid = m_ManipulationCommon->servoON();
	if (m_rid->id != 0){//Error
		std::cout << "Servo ON ERROR" << std::endl;
		std::cout << m_rid->comment << std::endl << std::endl;
	}
  //֐ߑx̐ݒ
  JARA_ARM::ULONG spdRation;
  spdRation = m_Speed;//%
  m_ManipulationMiddle->setSpeedJoint(spdRation);
  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t ManipulatorControler::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t ManipulatorControler::onExecute(RTC::UniqueId ec_id)
{
	/*T[{*/

	if (m_PointIn.isNew()){

		m_PointIn.read();
		JARA_ARM::CarPosWithElbow pos;
		//p͎ŌvZ邽ߏ̂
		//1
		pos.carPos[0][0] = cos(0);
		pos.carPos[1][0] = sin(0);
		pos.carPos[2][0] = 0.0;

		//2
		pos.carPos[0][1] = -sin(0);
		pos.carPos[1][1] = cos(0);
		pos.carPos[2][1] = 0.0;

		//3
		pos.carPos[0][2] = 0.0;
		pos.carPos[1][2] = 0.0;
		pos.carPos[2][2] = 1.0;

		//4ڏBm_PointIn(TimedPoint3D)xyzdoubleo
		pos.carPos[0][3] = m_Point.data.x;//[mm]
		pos.carPos[1][3] = m_Point.data.y;
		pos.carPos[2][3] = m_Point.data.z;

		m_rid = m_ManipulationMiddle->movePTPCartesianAbs(pos);
		if (m_rid->id != 0){//Error
			std::cout << "movePTPCartesianAbs ERROR" << std::endl;
			std::cout << m_rid->comment << std::endl << std::endl;
		}
	}

	if (m_HandIn.isNew()){
	    m_HandIn.read();
		if (m_Hand.data == 1){
			m_rid = m_ManipulationMiddle->closeGripper();
			if (m_rid->id != 0){//Error
				std::cout << "closeGripper ERROR" << std::endl;
				std::cout << m_rid->comment << std::endl << std::endl;
			}
		}
		else if (m_Hand.data == 0){
			m_rid = m_ManipulationMiddle->openGripper();
			if (m_rid->id != 0){//Error
				std::cout << "closeGripper ERROR" << std::endl;
				std::cout << m_rid->comment << std::endl << std::endl;
			}
		}
		
	}

	/*m_ManipulationCommon->servoON();

	m_ManipulationMiddle->movePTPCartesianAbs(m_PointIn);*/


  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t ManipulatorControler::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ManipulatorControler::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ManipulatorControler::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ManipulatorControler::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ManipulatorControler::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void ManipulatorControlerInit(RTC::Manager* manager)
  {
    coil::Properties profile(manipulatorcontroler_spec);
    manager->registerFactory(profile,
                             RTC::Create<ManipulatorControler>,
                             RTC::Delete<ManipulatorControler>);
  }
  
};